@@ -25,19 +25,24 @@ impl<T: fmt::Write> fmt::Write for WriteCounter<T> {
 //Panics if the element tag writing writes fail
 pub(super) fn render<T: Write>(
     mut writer: T,
-    mut plots: Vec<Plot>,
-    names: &dyn Names,
+    plotter:Plotter
 ) -> fmt::Result {
+    let Plotter {
+        names,
+        mut plots
+    } = plotter;
+
     write!(writer, "{}", moveable_format(|w| names.write_header(w)))?;
 
-    use crate::default_tags::*;
-    let width = WIDTH;
-    let height = HEIGHT;
+    write!(writer, "{}", moveable_format(|w| names.write_body(w)))?;
+    
+    let width = crate::WIDTH as f64;
+    let height = crate::HEIGHT as f64;
     let padding = 150.0;
     let paddingy = 100.0;
 
     let svg = &mut tagger::Element::new(&mut writer);
-
+    
     svg.single("rect", |w| {
         w.attr("class", "poloto_background")?
             .attr("fill", "white")?
@@ -47,10 +52,11 @@ pub(super) fn render<T: Write>(
             .attr("height", height)?
             .empty_ok()
     })?;
+    
 
     //Find range.
     let [minx, maxx, miny, maxy] =
-        if let Some(m) = util::find_bounds(plots.iter_mut().flat_map(|x| x.plots.iter_first())) {
+        if let Some(m) = util::find_bounds(plots.iter_mut().flat_map(|x| x.plots.iter_first().map(|[x,y]|[x as f64,y as f64]))) {
             m
         } else {
             //TODO test that this looks ok
@@ -254,8 +260,8 @@ pub(super) fn render<T: Write>(
 
         let it = plots.iter_second().map(|[x, y]| {
             [
-                padding + (x - minx) * scalex,
-                height - paddingy - (y - miny) * scaley,
+                padding + (x as f64 - minx) * scalex,
+                height - paddingy - (y as f64 - miny) * scaley,
             ]
         });
 
@@ -447,5 +453,8 @@ pub(super) fn render<T: Write>(
             .empty_ok()
     })?;
 
+
+    write!(writer, "{}", moveable_format(|w| names.write_footer(w)))?;
+
     Ok(())
 }
